"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deployExecutor = void 0;
const tslib_1 = require("tslib");
const devkit_1 = require("@nrwl/devkit");
const core_1 = require("@nx-extend/core");
const path_1 = require("path");
function deployExecutor(options, context) {
    var _a, _b;
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const { functionName, region, trigger = 'http', triggerValue = null, triggerEvent = null, triggerLocation = null, runtime = 'nodejs16', envVarsFile = null, maxInstances = 10, project = null, memory = '128MB', serviceAccount = null, entryPoint = null, retry = false, egressSettings = null, vpcConnector = null, securityLevel = null, secrets = [], gen = 1, concurrency = 1, cloudSqlInstance = null, timeout = null, cpu = 1 } = options;
        // If "recommended" option is selected set the currently recommended one of Google (https://cloud.google.com/functions/docs/concepts/nodejs-runtime)
        if (options.runtime === 'recommended') {
            options.runtime = 'nodejs18';
        }
        // Options with default values based of trigger type
        const { ingressSettings = trigger === 'http'
            ? null
            : 'internal-only', allowUnauthenticated = trigger === 'http' } = options;
        let correctMemory = memory;
        // Use the correct memory for the right gen
        if (gen === 2 && memory.endsWith('MB')) {
            correctMemory = memory.replace('MB', 'Mi');
        }
        if (triggerValue && trigger === 'http') {
            throw new Error('"triggerValue" is not accepted when trigger is "http"!');
        }
        const { targets } = context.workspace.projects[context.projectName];
        const validSecrets = secrets.map((secret) => {
            if (secret.includes('=') && secret.includes(':')) {
                return secret;
            }
            devkit_1.logger.warn(`"${secret}" is not a valid secret! It should be in the following format "ENV_VAR_NAME=SECRET:VERSION"`);
            return false;
        }).filter(Boolean);
        let gcloudCommand = 'gcloud';
        if (validSecrets.length > 0 && gen === 1) {
            devkit_1.logger.info('Using secrets, use gcloud beta');
            gcloudCommand = 'gcloud beta';
        }
        else if (gen === 2) {
            devkit_1.logger.info('Using gen 2, use gcloud alpha');
            gcloudCommand = 'gcloud alpha';
        }
        let { success } = (0, core_1.execCommand)((0, core_1.buildCommand)([
            `${gcloudCommand} functions deploy`,
            functionName,
            gen === 2 && '--gen2',
            `--trigger-${trigger}${triggerValue ? `=${triggerValue}` : ''}`,
            triggerEvent && `--trigger-event=${triggerEvent}`,
            triggerLocation && `--trigger-location=${triggerLocation}`,
            `--runtime=${runtime}`,
            `--memory=${correctMemory}`,
            `--region=${region}`,
            entryPoint && `--entry-point=${entryPoint}`,
            envVarsFile && `--env-vars-file=${envVarsFile}`,
            retry && `--retry`,
            ingressSettings && `--ingress-settings=${ingressSettings}`,
            egressSettings && `--egress-settings=${egressSettings}`,
            vpcConnector && `--vpc-connector=${vpcConnector}`,
            securityLevel && `--security-level=${securityLevel}`,
            timeout && `--timeout=${timeout}`,
            `--source=${(0, path_1.join)(context.root, (_b = (_a = targets === null || targets === void 0 ? void 0 : targets.build) === null || _a === void 0 ? void 0 : _a.options) === null || _b === void 0 ? void 0 : _b.outputPath.toString())}`,
            `--max-instances=${maxInstances}`,
            allowUnauthenticated && '--allow-unauthenticated',
            serviceAccount && `--service-account=${serviceAccount}`,
            validSecrets.length > 0 && `--set-secrets=${validSecrets.join(',')}`,
            project && `--project=${project}`,
            '--quiet'
        ]));
        if (success && gen === 2 && (concurrency > 0 || validSecrets.length > 0 || cloudSqlInstance)) {
            devkit_1.logger.info('Updating service with more configurations');
            const serviceUpdateCommand = (0, core_1.execCommand)((0, core_1.buildCommand)([
                `${gcloudCommand} run services update`,
                functionName,
                concurrency > 0 && `--concurrency ${concurrency}`,
                cpu && `--cpu ${cpu}`,
                cloudSqlInstance && `--add-cloudsql-instances=${cloudSqlInstance}`,
                `--region=${region}`,
                project && `--project=${project}`,
                '--quiet'
            ]));
            success = serviceUpdateCommand.success;
        }
        return Promise.resolve({ success });
    });
}
exports.deployExecutor = deployExecutor;
exports.default = deployExecutor;
//# sourceMappingURL=deploy.impl.js.map